@@ -320,6 +320,12 @@ static void callback(const nrsc5_event_t *evt, void *opaque)
     case NRSC5_EVENT_LOST_DEVICE:
         done_signal(st);
         break;
+    case NRSC5_EVENT_AGC:
+        if (evt->agc.is_final)
+            log_info("Best gain: %.1f dB, Peak amplitude: %.1f dBFS", evt->agc.gain_db, evt->agc.peak_dbfs);
+        else
+            log_debug("Gain: %.1f dB, Peak amplitude: %.1f dBFS", evt->agc.gain_db, evt->agc.peak_dbfs);
+        break;
     case NRSC5_EVENT_BER:
         dump_ber(evt->ber.cber);
         break;
@@ -350,6 +356,8 @@ static void callback(const nrsc5_event_t *evt, void *opaque)
         break;
     case NRSC5_EVENT_SYNC:
         log_info("Synchronized");
+        log_info("Frequency offset: %.0f Hz", evt->sync.freq_offset);
+        log_info("Primary service mode: %d", evt->sync.psmi);
         st->audio_ready = 0;
         break;
     case NRSC5_EVENT_LOST_SYNC:
@@ -398,16 +406,27 @@ static void callback(const nrsc5_event_t *evt, void *opaque)
         }
         break;
     case NRSC5_EVENT_STREAM:
-        log_info("Stream data: port=%04X seq=%04X mime=%08X size=%d", evt->stream.port, evt->stream.seq, evt->stream.component->data.mime, evt->stream.size);
+        log_debug("Stream data: port=%04X seq=%04X mime=%08X size=%d", evt->stream.component->data.port, evt->stream.seq, evt->stream.component->data.mime, evt->stream.size);
         break;
     case NRSC5_EVENT_PACKET:
-        log_info("Packet data: port=%04X seq=%04X mime=%08X size=%d", evt->packet.port, evt->packet.seq, evt->packet.component->data.mime, evt->packet.size);
+        log_debug("Packet data: port=%04X seq=%04X mime=%08X size=%d", evt->packet.component->data.port, evt->packet.seq, evt->packet.component->data.mime, evt->packet.size);
         break;
     case NRSC5_EVENT_LOT:
         if (st->aas_files_path)
             dump_aas_file(st, evt);
         strftime(time_str, sizeof(time_str), "%Y-%m-%dT%H:%M:%SZ", evt->lot.expiry_utc);
-        log_info("LOT file: port=%04X lot=%d name=%s size=%d mime=%08X expiry=%s", evt->lot.port, evt->lot.lot, evt->lot.name, evt->lot.size, evt->lot.mime, time_str);
+        log_info("LOT file: port=%04X lot=%d name=%s size=%d mime=%08X expiry=%s", evt->lot.component->data.port, evt->lot.lot, evt->lot.name, evt->lot.size, evt->lot.mime, time_str);
+        break;
+    case NRSC5_EVENT_LOT_HEADER:
+        strftime(time_str, sizeof(time_str), "%Y-%m-%dT%H:%M:%SZ", evt->lot.expiry_utc);
+        log_debug("LOT header: port=%04X lot=%d name=%s size=%d mime=%08X expiry=%s",
+                  evt->lot.component->data.port, evt->lot.lot, evt->lot.name, evt->lot.size, evt->lot.mime, time_str);
+        break;
+    case NRSC5_EVENT_LOT_FRAGMENT:
+        if (!evt->lot_fragment.is_duplicate)
+            log_debug("LOT fragment: port=%04X lot=%d seq=%d repeat=%d size=%d bytes_so_far=%d",
+                      evt->lot_fragment.component->data.port, evt->lot_fragment.lot, evt->lot_fragment.seq,
+                      evt->lot_fragment.repeat, evt->lot_fragment.size, evt->lot_fragment.bytes_so_far);
         break;
     case NRSC5_EVENT_STATION_ID:
         log_info("Country: %s, FCC facility ID: %d", evt->station_id.country_code, evt->station_id.fcc_facility_id);
@@ -461,16 +480,17 @@ static void callback(const nrsc5_event_t *evt, void *opaque)
             switch (evt->emergency_alert.location_format)
             {
             case NRSC5_LOCATION_FORMAT_SAME:
-                strcat(alert_details, "SAME=[");
+                strcat(alert_details, "SAME=");
                 break;
             case NRSC5_LOCATION_FORMAT_FIPS:
-                strcat(alert_details, "FIPS=[");
+                strcat(alert_details, "FIPS=");
                 break;
             case NRSC5_LOCATION_FORMAT_ZIP:
-                strcat(alert_details, "ZIP=[");
+                strcat(alert_details, "ZIP=");
                 break;
             }
 
+            strcat(alert_details, "[");
             for (i = 0; i < evt->emergency_alert.num_locations; i++)
             {
                 if (i > 0)
@@ -647,6 +667,7 @@ static int parse_args(state_t *st, int argc, char *argv[])
     st->bias_tee = 0;
     st->direct_sampling = -1;
     st->ppm_error = INT_MIN;
+    log_set_level(LOG_INFO);
 
     while ((opt = getopt_long(argc, argv, "r:w:o:t:d:p:g:ql:vH:TD:", long_opts, NULL)) != -1)
     {
