@@ -82,7 +82,7 @@ static int do_auto_gain(nrsc5_t *st)
         }
 
         float amplitude_db = 20 * log10f((max_sample - min_sample + 1) / 256.0f);
-        log_debug("Gain: %.1f dB, Peak amplitude: %.1f dBFS", gain / 10.0f, amplitude_db);
+        nrsc5_report_agc(st, gain / 10.0f, amplitude_db, 0);
 
         if (amplitude_db < -6.0f)
         {
@@ -102,7 +102,7 @@ static int do_auto_gain(nrsc5_t *st)
         }
     }
 
-    log_debug("Best gain: %.1f dB, Peak amplitude: %.1f dBFS", best_gain / 10.0f, best_amplitude_db);
+    nrsc5_report_agc(st, best_gain / 10.0f, best_amplitude_db, 1);
     st->gain = best_gain;
     set_tuner_gain(st, best_gain);
     ret = 0;
@@ -663,6 +663,17 @@ void nrsc5_report_lost_device(nrsc5_t *st)
     nrsc5_report(st, &evt);
 }
 
+void nrsc5_report_agc(nrsc5_t *st, float gain_db, float peak_dbfs, int is_final)
+{
+    nrsc5_event_t evt;
+
+    evt.event = NRSC5_EVENT_AGC;
+    evt.agc.gain_db = gain_db;
+    evt.agc.peak_dbfs = peak_dbfs;
+    evt.agc.is_final = is_final;
+    nrsc5_report(st, &evt);
+}
+
 void nrsc5_report_iq(nrsc5_t *st, const void *data, size_t count)
 {
     nrsc5_event_t evt;
@@ -673,11 +684,13 @@ void nrsc5_report_iq(nrsc5_t *st, const void *data, size_t count)
     nrsc5_report(st, &evt);
 }
 
-void nrsc5_report_sync(nrsc5_t *st)
+void nrsc5_report_sync(nrsc5_t *st, float freq_offset, int psmi)
 {
     nrsc5_event_t evt;
 
     evt.event = NRSC5_EVENT_SYNC;
+    evt.sync.freq_offset = freq_offset;
+    evt.sync.psmi = psmi;
     nrsc5_report(st, &evt);
 }
 
@@ -730,13 +743,13 @@ void nrsc5_report_ber(nrsc5_t *st, float cber)
     nrsc5_report(st, &evt);
 }
 
-void nrsc5_report_stream(nrsc5_t *st, uint16_t port, uint16_t seq, unsigned int size, const uint8_t *data,
+void nrsc5_report_stream(nrsc5_t *st, uint16_t seq, unsigned int size, const uint8_t *data,
                          nrsc5_sig_service_t *service, nrsc5_sig_component_t *component)
 {
     nrsc5_event_t evt;
 
     evt.event = NRSC5_EVENT_STREAM;
-    evt.stream.port = port;
+    evt.stream.port = component->data.port;
     evt.stream.seq = seq;
     evt.stream.size = size;
     evt.stream.mime = component->data.mime;
@@ -746,13 +759,13 @@ void nrsc5_report_stream(nrsc5_t *st, uint16_t port, uint16_t seq, unsigned int
     nrsc5_report(st, &evt);
 }
 
-void nrsc5_report_packet(nrsc5_t *st, uint16_t port, uint16_t seq, unsigned int size, const uint8_t *data,
+void nrsc5_report_packet(nrsc5_t *st, uint16_t seq, unsigned int size, const uint8_t *data,
                          nrsc5_sig_service_t *service, nrsc5_sig_component_t *component)
 {
     nrsc5_event_t evt;
 
     evt.event = NRSC5_EVENT_PACKET;
-    evt.packet.port = port;
+    evt.packet.port = component->data.port;
     evt.packet.seq = seq;
     evt.packet.size = size;
     evt.packet.mime = component->data.mime;
@@ -762,14 +775,14 @@ void nrsc5_report_packet(nrsc5_t *st, uint16_t port, uint16_t seq, unsigned int
     nrsc5_report(st, &evt);
 }
 
-void nrsc5_report_lot(nrsc5_t *st, uint16_t port, unsigned int lot, unsigned int size, uint32_t mime,
+void nrsc5_report_lot(nrsc5_t *st, int event, unsigned int lot, unsigned int size, uint32_t mime,
                       const char *name, const uint8_t *data, struct tm *expiry_utc,
                       nrsc5_sig_service_t *service, nrsc5_sig_component_t *component)
 {
     nrsc5_event_t evt;
 
-    evt.event = NRSC5_EVENT_LOT;
-    evt.lot.port = port;
+    evt.event = event;
+    evt.lot.port = component->data.port;
     evt.lot.lot = lot;
     evt.lot.size = size;
     evt.lot.mime = mime;
@@ -781,6 +794,25 @@ void nrsc5_report_lot(nrsc5_t *st, uint16_t port, unsigned int lot, unsigned int
     nrsc5_report(st, &evt);
 }
 
+void nrsc5_report_lot_fragment(nrsc5_t *st, unsigned int lot, unsigned int seq, unsigned int repeat, int is_duplicate,
+                               unsigned int size, unsigned int bytes_so_far, const uint8_t *data,
+                               nrsc5_sig_service_t *service, nrsc5_sig_component_t *component)
+{
+    nrsc5_event_t evt;
+
+    evt.event = NRSC5_EVENT_LOT_FRAGMENT;
+    evt.lot_fragment.lot = lot;
+    evt.lot_fragment.seq = seq;
+    evt.lot_fragment.repeat = repeat;
+    evt.lot_fragment.is_duplicate = is_duplicate;
+    evt.lot_fragment.size = size;
+    evt.lot_fragment.bytes_so_far = bytes_so_far;
+    evt.lot_fragment.data = data;
+    evt.lot_fragment.service = service;
+    evt.lot_fragment.component = component;
+    nrsc5_report(st, &evt);
+}
+
 static uint8_t convert_sig_component_type(uint8_t type)
 {
     switch (type)
