@@ -3,6 +3,7 @@ import ctypes
 import datetime
 import enum
 import math
+import os
 import platform
 import socket
 
@@ -37,6 +38,9 @@ class EventType(enum.Enum):
     DATA_SERVICE_DESCRIPTOR = 21
     EMERGENCY_ALERT = 22
     HERE_IMAGE = 23
+    LOT_HEADER = 24
+    LOT_FRAGMENT = 25
+    AGC = 26
 
 
 AUDIO_FRAME_SAMPLES = 2048
@@ -57,6 +61,12 @@ class ComponentType(enum.Enum):
     DATA = 1
 
 
+class AASType(enum.Enum):
+    STREAM = 0
+    PACKET = 1
+    LOT = 3
+
+
 class MIMEType(enum.Enum):
     PRIMARY_IMAGE = 0xBE4B7536
     STATION_LOGO = 0xD9C72536
@@ -174,6 +184,7 @@ class HEREImageType(enum.Enum):
 
 
 IQ = collections.namedtuple("IQ", ["data"])
+Sync = collections.namedtuple("Sync", ["freq_offset", "psmi"])
 MER = collections.namedtuple("MER", ["lower", "upper"])
 BER = collections.namedtuple("BER", ["cber"])
 HDC = collections.namedtuple("HDC", ["program", "data"])
@@ -190,6 +201,8 @@ SIG = collections.namedtuple("SIG", ["services"])
 STREAM = collections.namedtuple("STREAM", ["port", "seq", "mime", "data", "service", "component"])
 PACKET = collections.namedtuple("PACKET", ["port", "seq", "mime", "data", "service", "component"])
 LOT = collections.namedtuple("LOT", ["port", "lot", "mime", "name", "data", "expiry_utc", "service", "component"])
+LOTHeader = collections.namedtuple("LOTHeader", ["port", "lot", "mime", "name", "size", "expiry_utc", "service", "component"])
+LOTFragment = collections.namedtuple("LOTFragment", ["lot", "seq", "repeat", "bytes_so_far", "is_duplicate", "data", "service", "component"])
 SISAudioService = collections.namedtuple("SISAudioService", ["program", "access", "type", "sound_exp"])
 SISDataService = collections.namedtuple("SISDataService", ["access", "type", "mime_type"])
 SIS = collections.namedtuple("SIS", ["country_code", "fcc_facility_id", "name", "slogan", "message", "alert",
@@ -205,6 +218,7 @@ AudioService = collections.namedtuple("AudioService", ["program", "access", "typ
                                                        "digital_audio_gain", "common_delay", "latency"])
 HEREImage = collections.namedtuple("HEREImage", ["image_type", "seq", "n1", "n2", "time_utc", "latitude1", "longitude1",
                                                  "latitude2", "longitude2", "name", "data"])
+AGC = collections.namedtuple("AGC", ["gain_db", "peak_dbfs", "is_final"])
 
 
 class _IQ(ctypes.Structure):
@@ -214,6 +228,13 @@ class _IQ(ctypes.Structure):
     ]
 
 
+class _Sync(ctypes.Structure):
+    _fields_ = [
+        ("freq_offset", ctypes.c_float),
+        ("psmi", ctypes.c_int),
+    ]
+
+
 class _MER(ctypes.Structure):
     _fields_ = [
         ("lower", ctypes.c_float),
@@ -391,6 +412,20 @@ class _LOT(ctypes.Structure):
     ]
 
 
+class _LOTFragment(ctypes.Structure):
+    _fields_ = [
+        ("lot", ctypes.c_uint),
+        ("seq", ctypes.c_uint),
+        ("repeat", ctypes.c_uint),
+        ("size", ctypes.c_uint),
+        ("bytes_so_far", ctypes.c_uint),
+        ("is_duplicate", ctypes.c_int),
+        ("data", ctypes.POINTER(ctypes.c_char)),
+        ("service", ctypes.POINTER(_SIGService)),
+        ("component", ctypes.POINTER(_SIGComponent)),
+    ]
+
+
 class _SISAudioService(ctypes.Structure):
     pass
 
@@ -532,9 +567,18 @@ class _HEREImage(ctypes.Structure):
     ]
 
 
+class _AGC(ctypes.Structure):
+    _fields_ = [
+        ("gain_db", ctypes.c_float),
+        ("peak_dbfs", ctypes.c_float),
+        ("is_final", ctypes.c_int),
+    ]
+
+
 class _EventUnion(ctypes.Union):
     _fields_ = [
         ("iq", _IQ),
+        ("sync", _Sync),
         ("mer", _MER),
         ("ber", _BER),
         ("hdc", _HDC),
@@ -544,6 +588,7 @@ class _EventUnion(ctypes.Union):
         ("stream", _STREAM),
         ("packet", _PACKET),
         ("lot", _LOT),
+        ("lot_fragment", _LOTFragment),
         ("sis", _SIS),
         ("station_id", _StationID),
         ("station_name", _StationName),
@@ -555,6 +600,7 @@ class _EventUnion(ctypes.Union):
         ("emergency_alert", _EmergencyAlert),
         ("audio_service", _AudioService),
         ("here_image", _HEREImage),
+        ("agc", _AGC),
     ]
 
 
@@ -575,6 +621,7 @@ class NRSC5:
     def _load_library(self):
         if NRSC5.libnrsc5 is None:
             if platform.system() == "Windows":
+                dll_dir = os.add_dll_directory(os.path.dirname(__file__))
                 lib_name = "libnrsc5.dll"
             elif platform.system() == "Linux":
                 lib_name = "libnrsc5.so"
@@ -583,6 +630,8 @@ class NRSC5:
             else:
                 raise NRSC5Error("Unsupported platform: " + platform.system())
             NRSC5.libnrsc5 = ctypes.cdll.LoadLibrary(lib_name)
+            if platform.system() == "Windows":
+                dll_dir.close()
             self.radio = ctypes.c_void_p()
 
     @staticmethod
@@ -615,6 +664,9 @@ class NRSC5:
         if evt_type == EventType.IQ:
             iq = c_evt.u.iq
             evt = IQ(iq.data[:iq.count])
+        elif evt_type == EventType.SYNC:
+            sync = c_evt.u.sync
+            evt = Sync(sync.freq_offset, sync.psmi)
         elif evt_type == EventType.MER:
             mer = c_evt.u.mer
             evt = MER(mer.lower, mer.upper)
@@ -669,7 +721,7 @@ class NRSC5:
                     if component_type == ComponentType.DATA:
                         data = SIGDataComponent(component.u.data.port,
                                                 ServiceDataType(component.u.data.service_data_type),
-                                                component.u.data.type, MIMEType(component.u.data.mime))
+                                                AASType(component.u.data.type), MIMEType(component.u.data.mime))
                     component_obj = SIGComponent(component_type, component.id, audio, data)
                     if component_type == ComponentType.AUDIO:
                         audio_component = component_obj
@@ -685,18 +737,29 @@ class NRSC5:
             stream = c_evt.u.stream
             service = self.services[stream.service.contents.number]
             component = self.components[(stream.service.contents.number, stream.component.contents.id)]
-            evt = STREAM(stream.port, stream.seq, MIMEType(component.data.mime), stream.data[:stream.size], service, component)
+            evt = STREAM(component.data.port, stream.seq, MIMEType(component.data.mime), stream.data[:stream.size], service, component)
         elif evt_type == EventType.PACKET:
             packet = c_evt.u.packet
             service = self.services[packet.service.contents.number]
             component = self.components[(packet.service.contents.number, packet.component.contents.id)]
-            evt = PACKET(packet.port, packet.seq, MIMEType(component.data.mime), packet.data[:packet.size], service, component)
+            evt = PACKET(component.data.port, packet.seq, MIMEType(component.data.mime), packet.data[:packet.size], service, component)
         elif evt_type == EventType.LOT:
             lot = c_evt.u.lot
             service = self.services[lot.service.contents.number]
             component = self.components[(lot.service.contents.number, lot.component.contents.id)]
             expiry_time = self._timestruct_to_datetime(lot.expiry_utc)
-            evt = LOT(lot.port, lot.lot, MIMEType(lot.mime), self._decode(lot.name), lot.data[:lot.size], expiry_time, service, component)
+            evt = LOT(component.data.port, lot.lot, MIMEType(lot.mime), self._decode(lot.name), lot.data[:lot.size], expiry_time, service, component)
+        elif evt_type == EventType.LOT_HEADER:
+            lot = c_evt.u.lot
+            service = self.services[lot.service.contents.number]
+            component = self.components[(lot.service.contents.number, lot.component.contents.id)]
+            expiry_time = self._timestruct_to_datetime(lot.expiry_utc)
+            evt = LOTHeader(component.data.port, lot.lot, MIMEType(lot.mime), self._decode(lot.name), lot.size, expiry_time, service, component)
+        elif evt_type == EventType.LOT_FRAGMENT:
+            lot_fragment = c_evt.u.lot_fragment
+            service = self.services[lot_fragment.service.contents.number]
+            component = self.components[(lot_fragment.service.contents.number, lot_fragment.component.contents.id)]
+            evt = LOTFragment(lot_fragment.lot, lot_fragment.seq, lot_fragment.repeat, lot_fragment.bytes_so_far, bool(lot_fragment.is_duplicate), lot_fragment.data[:lot_fragment.size], service, component)
         elif evt_type == EventType.SIS:
             sis = c_evt.u.sis
 
@@ -791,6 +854,9 @@ class NRSC5:
                 self._decode(here_image.name),
                 here_image.data[:here_image.size]
             )
+        elif evt_type == EventType.AGC:
+            agc = c_evt.u.agc
+            evt = AGC(agc.gain_db, agc.peak_dbfs, bool(agc.is_final))
 
         self.callback(evt_type, evt, *self.callback_args)
 
